import { Button, Card, Col, Form, Result, Row, Spin, Switch } from "antd";
import { useTranslation } from "react-i18next";
import {
  Constants,
  getUserSessionFromLocalStorage,
  myFetch,
} from "../../../utils";
import { useEffect, useRef, useState } from "react";
import MyCenteredContainer from "../../../Components/MyContainer";
import {
  MyCalendarMaxFutureBookingDaysFormInput,
  MyCalendarMinEarliestBookingTimeFormInput,
} from "../../../Components/MyFormInputs";
import {
  RequestState,
  RequestStateItem,
} from "../../../Components/MyRequestStateItem";

export default function StoreCalendar() {
  const { t } = useTranslation();

  const [calendarSettings, setCalendarSettings] = useState({});
  const [isRequesting, setIsRequesting] = useState(true);

  useEffect(() => {
    // delete session cookie
    document.cookie = `session=; path=/; expires=Thu, 01 Jan 1970 00:00:01 GMT;`;

    myFetch("/calendar/settings", "GET")
      .then((res) => {
        setIsRequesting(false);
        setCalendarSettings(res);
      })
      .catch((err) => {
        console.log(err);
      });
  }, []);

  if (isRequesting) {
    return (
      <MyCenteredContainer>
        <Spin size="large" />
      </MyCenteredContainer>
    );
  }

  return (
    <>
      {calendarSettings.connected === false ? (
        <MyCenteredContainer>
          <Result
            status="warning"
            title={t("calendar.googleCalendarNotConnected.title")}
            subTitle={t("calendar.googleCalendarNotConnected.subTitle")}
            extra={
              <Button
                type="primary"
                onClick={() => {
                  document.cookie = `session=${getUserSessionFromLocalStorage()}; path=/;`;

                  window.location.href = `${Constants.API_ADDRESS}/calendar/auth/google`;
                }}
              >
                {t("calendar.googleCalendarNotConnected.button")}
              </Button>
            }
          />
        </MyCenteredContainer>
      ) : (
        <>
          <h1>{t("calendar.pageTitle")}</h1>

          <Row gutter={[16, 16]}>
            <Col xs={24} md={12}>
              <CardPersonalCalendarSettings
                settings={calendarSettings.userSettings}
              />
            </Col>
            <Col xs={24} md={12}>
              {calendarSettings.storeSettings && (
                <CardStoreCalendarSettings
                  settings={calendarSettings.storeSettings}
                />
              )}
            </Col>
          </Row>
        </>
      )}
    </>
  );
}

function CardPersonalCalendarSettings({ settings }) {
  const { t } = useTranslation();
  const [form] = Form.useForm();

  const [requestState, setRequestState] = useState(RequestState.INIT);
  const delayTimeout = useRef();

  const usingPrimaryCalendar = Form.useWatch(
    "calendarUsingPrimaryCalendar",
    form
  );

  useEffect(() => {
    if (!settings) return;

    form.setFieldValue(
      "calendarUsingPrimaryCalendar",
      settings.calendar_using_primary_calendar
    );
  }, []);

  useEffect(() => {
    if (usingPrimaryCalendar === undefined) return;

    if (RequestState.INIT === requestState) {
      setRequestState(RequestState.NOTHING);
      return;
    }

    setRequestState(RequestState.REQUESTING);

    if (delayTimeout.current) {
      clearTimeout(delayTimeout.current);
    }

    delayTimeout.current = setTimeout(() => {
      myFetch("/calendar/settings/personal", "POST", {
        calendarUsingPrimaryCalendar: usingPrimaryCalendar,
      })
        .then(() => setRequestState(RequestState.SUCCESS))
        .catch((errStatus) => {
          console.log(errStatus);
          setRequestState(RequestState.FAILED);
        });
    }, 500);
  }, [usingPrimaryCalendar]);

  return (
    <Card
      title={t("calendar.cardPersonalCalendarSettings.title")}
      extra={
        <RequestStateItem
          state={requestState}
          setRequestState={setRequestState}
        />
      }
    >
      <Form form={form} requiredMark={false}>
        <Form.Item
          name="calendarUsingPrimaryCalendar"
          label={t("common.calendarUsingPrimaryCalendar")}
          valuePropName="checked"
        >
          <Switch />
        </Form.Item>
      </Form>
    </Card>
  );
}

function CardStoreCalendarSettings({ settings }) {
  const { t } = useTranslation();
  const [form] = Form.useForm();

  const [requestState, setRequestState] = useState(RequestState.INIT);
  const delayTimeout = useRef();

  const calendarMaxFutureBookingDays = Form.useWatch(
    "calendarMaxFutureBookingDays",
    form
  );

  const calendarMinEarliestBookingTime = Form.useWatch(
    "calendarMinEarliestBookingTime",
    form
  );

  useEffect(() => {
    if (!settings) return;

    form.setFieldsValue({
      calendarMaxFutureBookingDays: settings.calendar_max_future_booking_days,
      calendarMinEarliestBookingTime:
        settings.calendar_min_earliest_booking_time,
    });
  }, []);

  useEffect(() => {
    // undefined on first render
    // null when input is empty
    if (
      calendarMaxFutureBookingDays === undefined ||
      calendarMinEarliestBookingTime === undefined ||
      calendarMaxFutureBookingDays === null ||
      calendarMinEarliestBookingTime === null
    )
      return;

    if (RequestState.INIT === requestState) {
      setRequestState(RequestState.NOTHING);
      return;
    }

    setRequestState(RequestState.REQUESTING);

    if (delayTimeout.current) {
      clearTimeout(delayTimeout.current);
    }

    delayTimeout.current = setTimeout(() => {
      myFetch("/calendar/settings/store", "POST", {
        calendarMaxFutureBookingDays,
        calendarMinEarliestBookingTime,
      })
        .then(() => setRequestState(RequestState.SUCCESS))
        .catch((errStatus) => {
          console.log(errStatus);
          setRequestState(RequestState.FAILED);
        });
    }, 500);
  }, [calendarMaxFutureBookingDays, calendarMinEarliestBookingTime]);

  return (
    <Card
      title={t("calendar.cardStoreCalendarSettings.title")}
      extra={
        <RequestStateItem
          state={requestState}
          setRequestState={setRequestState}
        />
      }
    >
      <Form form={form} requiredMark={false}>
        <>
          <MyCalendarMaxFutureBookingDaysFormInput />

          <MyCalendarMinEarliestBookingTimeFormInput />
        </>
      </Form>
    </Card>
  );
}
